/*********************************************************************************

Copyright(c) 2005 Analog Devices, Inc. All Rights Reserved. 

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.  

$RCSfile: ezkitutilities.c,v $
$Revision: 1.17 $
$Date: 2005/11/11 22:15:29 $

Description:	EZ-Kit utility routines.  This file contains a collection
				of functions that automate typical EZ-Kit functionality, 
				including control of LEDs and push buttons.  
			
*********************************************************************************/


/*********************************************************************

Include files

*********************************************************************/

#include <services/services.h>		// system service includes
#include <sysreg.h>					// system config definitions
#include "ezkitutilities.h"			// EZ-Kit utility definitions 



/*********************************************************************

Button/LED to Flag pin mappings

*********************************************************************/

#if defined(__ADSP_EDINBURGH__)			// ADSP-BF533 EZ-Kit specific info
ADI_FLAG_ID Button[] = {
	/* button[0] (PB1) */ ADI_FLAG_PF8, 	// EZ_FIRST_BUTTON
	/* button[1] (PB2) */ ADI_FLAG_PF9, 
	/* button[2] (PB3) */ ADI_FLAG_PF10, 
	/* button[3] (PB4) */ ADI_FLAG_PF11		// EZ_LAST_BUTTON
};
ADI_FLAG_ID LED[] = {	//(not used on Edinburgh but needed as placeholder)
	/* led[4] */ ADI_FLAG_PF0, 				// EZ_FIRST_LED
	/* led[5] */ ADI_FLAG_PF0,
	/* led[6] */ ADI_FLAG_PF0,
	/* led[7] */ ADI_FLAG_PF0,
	/* led[8] */ ADI_FLAG_PF0,
	/* led[9] */ ADI_FLAG_PF0				// EZ_LAST_LED
};
#endif

#if defined(__ADSP_BRAEMAR__)			// ADSP-BF537 EZ-Kit specific info
ADI_FLAG_ID Button[] = {
	/* button[0] (PB1) */ ADI_FLAG_PF2, 	// EZ_FIRST_BUTTON
	/* button[1] (PB2) */ ADI_FLAG_PF3, 
	/* button[2] (PB3) */ ADI_FLAG_PF4, 
	/* button[3] (PB4) */ ADI_FLAG_PF5		// EZ_LAST_BUTTON
};
ADI_FLAG_ID LED[] = {
	/* led[1] */ ADI_FLAG_PF6, 				// EZ_FIRST_LED
	/* led[2] */ ADI_FLAG_PF7, 
	/* led[3] */ ADI_FLAG_PF8, 
	/* led[4] */ ADI_FLAG_PF9, 
	/* led[5] */ ADI_FLAG_PF10, 
	/* led[6] */ ADI_FLAG_PF11				// EZ_LAST_LED
};
#endif


#if defined(__ADSP_TETON__)
ADI_FLAG_ID LED[] = {
	/* led[0]  (LED5)  */ ADI_FLAG_PF40, 	// EZ_FIRST_LED
	/* led[1]  (LED6)  */ ADI_FLAG_PF41, 
	/* led[2]  (LED7)  */ ADI_FLAG_PF42, 
	/* led[3]  (LED8)  */ ADI_FLAG_PF43, 
	/* led[4]  (LED9)  */ ADI_FLAG_PF44, 
	/* led[5]  (LED10) */ ADI_FLAG_PF45, 
	/* led[6]  (LED11) */ ADI_FLAG_PF46, 
	/* led[7]  (LED12) */ ADI_FLAG_PF47,
	/* led[8]  (LED13) */ ADI_FLAG_PF32, 
	/* led[9]  (LED14) */ ADI_FLAG_PF33, 
	/* led[10] (LED15) */ ADI_FLAG_PF34, 
	/* led[11] (LED16) */ ADI_FLAG_PF35, 
	/* led[12] (LED17) */ ADI_FLAG_PF36, 
	/* led[13] (LED18) */ ADI_FLAG_PF37, 
	/* led[14] (LED19) */ ADI_FLAG_PF38, 
	/* led[15] (LED20) */ ADI_FLAG_PF39		// EZ_LAST_LED
};

ADI_FLAG_ID Button[] = {
	/* button[0] (SW6) */ ADI_FLAG_PF5, 	// EZ_FIRST_BUTTON
	/* button[1] (SW7) */ ADI_FLAG_PF6, 
	/* button[2] (SW8) */ ADI_FLAG_PF7, 
	/* button[3] (SW9) */ ADI_FLAG_PF8		// EZ_LAST_BUTTON
};

#endif


#define EZ_NUM_BUTTONS					(sizeof(Button)/sizeof(Button[0]))		// number of buttons
#define EZ_BUTTON_TO_FLAG(ButtonNumber) Button[ButtonNumber-EZ_FIRST_BUTTON]	// converts button number to flag ID

#define EZ_NUM_LEDS						(sizeof(LED)/sizeof(LED[0]))			// number of LEDs
#define EZ_LED_TO_FLAG(LEDNumber) 		LED[LEDNumber-EZ_FIRST_LED]				// converts LED number to flag ID


/*********************************************************************

Peripheral ID Macros

This program works on the EZ-Kits.  The macros
below are used to identify which EZ-Kit we're targeting.  Specifically, 
the FLAG_PERIPHERAL_ID macro is set to the peripheral ID to which the 
interrupt driven push buttons are mapped.  See the adi_int.h file
within the system services library (blackfin/include/services) for
more information on peripheral IDs.  

*********************************************************************/

#if defined(__ADSP_EDINBURGH__)
#define FLAG_PERIPHERAL_ID	(ADI_INT_PFA)
#endif

#if defined(__ADSP_BRAEMAR__)
#define FLAG_PERIPHERAL_ID	(ADI_INT_PORTFG_A)
#endif

#if defined(__ADSP_TETON__)
#define FLAG_PERIPHERAL_ID	(ADI_INT_PF0_15_A)
#endif


/*********************************************************************

Enumerations and defines

*********************************************************************/

#if defined(__ADSP_EDINBURGH__)			// the Edinburgh EZ-Kit uses a flash for ports

#define pFlashA_PortA_In	((volatile unsigned char *)0x20270000)	// address of flash A port A input data register
#define pFlashA_PortA_Out	((volatile unsigned char *)0x20270004)	// address of flash A port A output data register
#define pFlashA_PortA_Dir	((volatile unsigned char *)0x20270006)	// address of flash A port A direction register

#define pFlashA_PortB_In	((volatile unsigned char *)0x20270001)	// address of flash A port B input data register
#define pFlashA_PortB_Out	((volatile unsigned char *)0x20270005)	// address of flash A port B output data register
#define pFlashA_PortB_Dir	((volatile unsigned char *)0x20270007)	// address of flash A port B direction register

#define pFlashB_PortA_In	((volatile unsigned char *)0x202E0000)	// address of flash B port A input data register
#define pFlashB_PortA_Out	((volatile unsigned char *)0x202E0004)	// address of flash B port A output data register
#define pFlashB_PortA_Dir	((volatile unsigned char *)0x202E0006)	// address of flash B port A direction register

#define pFlashB_PortB_In	((volatile unsigned char *)0x202E0001)	// address of flash B port B input data register
#define pFlashB_PortB_Out	((volatile unsigned char *)0x202E0005)	// address of flash B port B output data register
#define pFlashB_PortB_Dir	((volatile unsigned char *)0x202E0007)	// address of flash B port B direction register

#endif


/*********************************************************************

Miscellaneous defines

*********************************************************************/

#if defined(__ADSP_EDINBURGH__)			// ADSP-BF533 EZ-Kit specific info
#define RST_7183 				(0x8) 	    //decoder reset bit #3 in flashA portA
#define PPICLK_ADV7183_SELECT 	(0x10)		//decoder clock to PPI bit #4 in flashA portA
#define ADV7183_OE_MASK 	(0x4)		//ADV7183 /OE = PF2
#define ADV7183_OE			(0)			//Index into Codec flag handler array
#define RST_7171 				(0x4)		// encoder reset
#endif


#if defined(__ADSP_TETON__)
#define ADV7183_RESET		(13)		//decoder reset bit
#define ADV7183_OE 		(2)		//ADV7183 /OE = PF2
#define ADV7183_Reset_Delay   	0x04000000	// For delay after reset
#define ADV7183_Lock   		0x0000ffff	// For startup locktime delay
#define ADV7179_RESET 		(14)		// encoder reset bit
#endif 

/*********************************************************************

	Function:		ezInit

	Description:	Initializes the Utilities service.  Specifically:
					- configures async memories
					- configures flash (where applicable)
					- configures power to 600/120
					- configures all LEDs
					- configures all pushbuttons
					
*********************************************************************/
void ezInit(u8 NumCores)
{
	
	// configure async memory
#if defined(__ADSP_EDINBURGH__)			// ADSP-BF533 EZ-Kit specific info
	ezInitAsync();
#endif
	
	// configure flash 
#if defined(__ADSP_EDINBURGH__)			// ADSP-BF533 EZ-Kit specific info
	ezInitFlashA();
#endif
	
	// configure power
	ezInitPower(NumCores);

	// configure LEDs
	ezInitLEDs();
	
	// configure buttons
	ezInitButtons();
	
}

/*********************************************************************

	Function:		ezTerminate

	Description:	Terminates the utilities.  Specifically
					- terminates LEDs
					- terminates buttons

		
*********************************************************************/
void ezTerminate()
{
	// terminate LEDs
	ezTerminateLEDs();
	
	// terminate buttons
	ezTerminateButtons();
}




/*********************************************************************

	Function:		ezEnableVideoDecoder

	Description:	Connects up the bus, sets up the video clocking and
					enables the AD7183.  

*********************************************************************/

void ezEnableVideoDecoder(void)			// connects bus, sets up video clock and enables the 7183
{

#if defined(__ADSP_EDINBURGH__)			// ADSP-BF533 EZ-Kit specific info
#define ADV7183_OE_FLAG ADI_FLAG_PF2
	
	unsigned short tempReg;
	
	//The GPIOs of the flash are linked to the ADV7183 reset pin as well as to the 
	//switch that drives the PPI clock. De-assert the ADV7183 reset and link the ADV7183 
	//Clock "LLC1" to the Blackfin PPI clock input.  See EZ-KIT schematics for more details.
	tempReg = *pFlashA_PortA_Out;
	*pFlashA_PortA_Out = tempReg | RST_7183 | PPICLK_ADV7183_SELECT;
	
	//Setup the Blackfin PF registers
#if defined(__ADSP_EDINBURGH__) && (__SILICON_REVISION__==0xffff || __SILICON_REVISION__==0x1)
 		tempReg = *pFIO_INEN;
		*pFIO_INEN = tempReg | ADV7183_OE_MASK;
#endif
	
	// open PF2 and PF13
	adi_flag_Open(ADI_FLAG_PF2);
	adi_flag_Open(ADI_FLAG_PF13);
	
	//Blackfin PF2 pin must be set as output
	adi_flag_SetDirection(ADI_FLAG_PF2, ADI_FLAG_DIRECTION_OUTPUT);
	
	//Set the Blackfin pin PF2 to output enable the ADV7183 data bus	
	adi_flag_Clear(ADI_FLAG_PF2);
	
#endif // Edinburgh


#if defined(__ADSP_TETON__)

 	volatile long delay_time;	

	//Blackfin PF2 pin must be set as output
	adi_flag_SetDirection(ADI_FLAG_PF2, ADI_FLAG_DIRECTION_OUTPUT);

	//Blackfin PF13 pin must be set as output
	adi_flag_SetDirection(ADI_FLAG_PF13, ADI_FLAG_DIRECTION_OUTPUT);
	ssync();
	
	// set bit PF2 to disable ADV7183 outputs
	adi_flag_Set(ADI_FLAG_PF2);
	ssync();

	// clear bit PF13 to reset ADV7183 
	adi_flag_Clear(ADI_FLAG_PF13);
	ssync();

	// set bit PF13 to re-enable ADV7183 
	adi_flag_Set(ADI_FLAG_PF13);
	ssync();

	for (delay_time = 0; delay_time < ADV7183_Reset_Delay; delay_time++)
		asm("nop;");
	
	// clear bit to enable ADV7183 outputs
	adi_flag_Clear(ADI_FLAG_PF2);
	ssync();

	for (delay_time = 0; delay_time < ADV7183_Lock; delay_time++)
		asm("nop;");
				 
#endif   // Teton

}

/*********************************************************************

	Function:		ezEnableVideoEncoder

	Description:	Enables the AD7171 or AD7179

*********************************************************************/

void ezEnableVideoEncoder(void)		// enables the 7171
{

#if defined(__ADSP_EDINBURGH__)			// ADSP-BF533 EZ-Kit specific info
	*pFlashA_PortA_Out = 0;
	*pFlashA_PortA_Out |= RST_7171;
#endif // Edinburgh


#if defined(__ADSP_TETON__)

	// open PF14
	adi_flag_Open(ADI_FLAG_PF14);
	
	//Blackfin PF14 pin must be set as output
	adi_flag_SetDirection(ADI_FLAG_PF14, ADI_FLAG_DIRECTION_OUTPUT);
	ssync();
 
	// clear bit to reset ADV7179, Blackfin pin PF14 	
	adi_flag_Clear(ADI_FLAG_PF14);
	ssync();

	adi_flag_Set(ADI_FLAG_PF14);
	ssync();


#endif   // Teton

}




/*********************************************************************

	Function:	ezDisableVideoEncoder

	Description:	Disables the AD7179.  

*********************************************************************/

void ezDisableVideoEncoder(void)
{

#if defined(__ADSP_TETON__)
	adi_flag_Clear(ADI_FLAG_PF14);

	// close PF14
	adi_flag_Close(ADI_FLAG_PF14);
	
	ssync();
#endif
}


/*********************************************************************

	Function:		ezDisableVideoDecoder

	Description:	 Disables 7183
*********************************************************************/

void ezDisableVideoDecoder(void)
{

#if defined(__ADSP_EDINBURGH__)			// ADSP-BF533 EZ-Kit specific info
	// do we need to do anything???
#endif // Edinburgh

#if defined(__ADSP_TETON__)

	// set bit PF2 to disable ADV7183 outputs
	adi_flag_Set(ADI_FLAG_PF2);
	ssync();

	// clear bit PF13 to reset ADV7183 
	adi_flag_Clear(ADI_FLAG_PF13);
	ssync();	
#endif

	// close PF2 and PF13
	adi_flag_Close(ADI_FLAG_PF2);
	adi_flag_Close(ADI_FLAG_PF13);
	
}


/*********************************************************************

	Function:		ezTurnOnLED

	Description:	Turns an LED on.  

*********************************************************************/

void ezTurnOnLED(u32 LEDNumber)	// turns on an LED
{

#if defined(__ADSP_EDINBURGH__)			// ADSP-BF533 EZ-Kit specific info
	switch(LEDNumber) {
		case 4:
			*pFlashA_PortB_Out |= 0x0001;
			break;
		case 5:
			*pFlashA_PortB_Out |= 0x0002;
			break;
		case 6:
			*pFlashA_PortB_Out |= 0x0004;
			break;
		case 7:
			*pFlashA_PortB_Out |= 0x0008;
			break;
		case 8:
			*pFlashA_PortB_Out |= 0x0010;
			break;
		case 9:
			*pFlashA_PortB_Out |= 0x0020;
			break;
	}
#endif // Edinburgh

#if defined(__ADSP_BRAEMAR__) || defined (__ADSP_TETON__) // ADSP-BF537 & BF561 EZ-Kit specific info
	adi_flag_Set(EZ_LED_TO_FLAG(LEDNumber));
#endif // Braemar or teton

}
			


/*********************************************************************

	Function:		ezTurnOffLED

	Description:	Turns an LED off.  

*********************************************************************/

void ezTurnOffLED(u32 LEDNumber)	// turns on an LED
{

#if defined(__ADSP_EDINBURGH__)			// ADSP-BF533 EZ-Kit specific info
	switch(LEDNumber) {
		case 4:
			*pFlashA_PortB_Out &= ~0x0001;
			break;
		case 5:
			*pFlashA_PortB_Out &= ~0x0002;
			break;
		case 6:
			*pFlashA_PortB_Out &= ~0x0004;
			break;
		case 7:
			*pFlashA_PortB_Out &= ~0x0008;
			break;
		case 8:
			*pFlashA_PortB_Out &= ~0x0010;
			break;
		case 9:
			*pFlashA_PortB_Out &= ~0x0020;
			break;
	}
#endif // Edinburgh

#if defined(__ADSP_BRAEMAR__) || defined (__ADSP_TETON__) // ADSP-BF537 & BF561 EZ-Kit specific info
	adi_flag_Clear(EZ_LED_TO_FLAG(LEDNumber));
#endif // Braemar or Teton

}
			

/*********************************************************************

	Function:		ezToggleLED

	Description:	Toggles the state of an LED.

*********************************************************************/

void ezToggleLED(u32 LEDNumber)	// toggles an LED
{
	if (ezIsLEDOn(LEDNumber)) {
		ezTurnOffLED(LEDNumber);
	} else {
		ezTurnOnLED(LEDNumber);
	}
}




/*********************************************************************

	Function:		ezTurnOnAllLEDs

	Description:	Turns on all LEDs.  

*********************************************************************/

void ezTurnOnAllLEDs(void)					// turns on all LEDs
{
	int i;
	
	// turn them on
	for (i = EZ_FIRST_LED; i <= EZ_LAST_LED; i++) {
		ezTurnOnLED(i);
	}

}
	

/*********************************************************************

	Function:		ezTurnOffAllLEDs

	Description:	Turns off all LEDs.  

*********************************************************************/

void ezTurnOffAllLEDs(void)					// turns off all LEDs
{
	int i;
	
	// turn them off
	for (i = EZ_FIRST_LED; i <= EZ_LAST_LED; i++) {
		ezTurnOffLED(i);
	}

}
	

/*********************************************************************

	Function:		ezCycleLEDs

	Description:	Cycles through each LED in turn.  

*********************************************************************/

void ezCycleLEDs(void)						// cycles through LEDS
{												// changes each time called
static int LED = EZ_FIRST_LED;

			ezTurnOffLED(LED);
			LED++;
			if (LED > EZ_LAST_LED) LED = EZ_FIRST_LED;
			ezTurnOnLED(LED);
}
	

/*********************************************************************

	Function:		ezIsLEDOn

	Description:	Returns TRUE if an LED is lit, FALSE otherwise.

*********************************************************************/

u32 ezIsLEDOn(u32 LEDNumber)	// test to see if an LED is lit
{

#if defined(__ADSP_EDINBURGH__)			// ADSP-BF533 EZ-Kit specific info
	switch(LEDNumber) {
		case 4:
			if (*pFlashA_PortB_In & 0x0001) return (TRUE);
			break;
		case 5:
			if (*pFlashA_PortB_In & 0x0002) return (TRUE);
			break;
		case 6:
			if (*pFlashA_PortB_In & 0x0004) return (TRUE);
			break;
		case 7:
			if (*pFlashA_PortB_In & 0x0008) return (TRUE);
			break;
		case 8:
			if (*pFlashA_PortB_In & 0x0010) return (TRUE);
			break;
		case 9:
			if (*pFlashA_PortB_In & 0x0020) return (TRUE);
			break;
	}

	// return
	return (FALSE);
#endif

#if defined(__ADSP_BRAEMAR__) || defined (__ADSP_TETON__) // ADSP-BF537 & BF561 EZ-Kit specific info

	u32	Value;	
	  
	adi_flag_Sense(EZ_LED_TO_FLAG(LEDNumber), &Value);

	// return
	return (Value );
 
#endif // Braemar or Teton

}

/*********************************************************************

	Function:		ezGetDisplay

	Description:	Returns the current status of the LED display

*********************************************************************/
u32 ezGetDisplay()
{
	int i;
	u32 Mask;
	u32 Display;
	
	// assume nothing's on and set our mask
	Display = 0;
	Mask = 1;
	
	// FOR (each LED)
	for (i = EZ_FIRST_LED; i <= EZ_LAST_LED; i++, Mask <<= 1) {
		
		// set the bit in Display if the LED is on
		if (ezIsLEDOn(i)) Display |= Mask;
		
	// ENDFOR
	}
	
	// return
	return (Display);
}

/*********************************************************************

	Function:		ezSetDisplay

	Description:	Sets the LED display

*********************************************************************/
void ezSetDisplay(u32 Display)
{

	int i;
	
	// FOR (each LED)
	for (i = EZ_FIRST_LED; i <= EZ_LAST_LED; i++, Display >>= 1) {
		
		// light the LED if that bit in the display is set
		if (Display & 0x1) {
			ezTurnOnLED(i);
		} else {
			ezTurnOffLED(i);
		}
		
	// ENDFOR
	}
	
}



/*********************************************************************

	Function:		ezErrorCheck

	Description:	This function is intended to be used as a means to 
					quickly determine if a function has returned a non-zero
					(hence an error) return code.  All driver and system
					services functions return a value of zero for success and 
					a non-zero value when a failure occurs.  This function
					lights all the LEDs and spins when a non-zero value is 
					passed to it.  

*********************************************************************/

void ezErrorCheck(u32 Result)				// lights all LEDS and spins on nonzero value
{
	while (Result != 0) {
		ezTurnOnAllLEDs();
	}
}
			
	



/*********************************************************************

	Function:		ezIsButtonPushed

	Description:	Returns TRUE if a button has been pushed, FALSE otherwise.

*********************************************************************/

u32 ezIsButtonPushed(u32 ButtonNumber) 		// returns TRUE if button is pushed, FALSE otherwise
{
	u32	Value;
	
	// sense it
	adi_flag_Sense(EZ_BUTTON_TO_FLAG(ButtonNumber), &Value);

	// return
	return (Value);
	
}		


/*********************************************************************

	Function:		ezEnableButtonCallback

	Description:	Enables generation of a callback in response to
					a button being pushed.

*********************************************************************/

u32	ezEnableButtonCallback		(					// enables generation of a push button callback
	u32 				ButtonNumber,
	void				*ClientHandle,
	ADI_DCB_HANDLE		DCBHandle,
	ADI_DCB_CALLBACK_FN	ClientCallback
) {

	// use the flag service to hook the interrupt
	return((u32)adi_flag_InstallCallback(EZ_BUTTON_TO_FLAG(ButtonNumber), FLAG_PERIPHERAL_ID, ADI_FLAG_TRIGGER_RISING_EDGE, TRUE, ClientHandle, DCBHandle, ClientCallback));
	
	// return
}



/*********************************************************************

	Function:		ezDisableButtonCallback

	Description:	Disables generation of a callback in response to 
					a button being pushed.

*********************************************************************/

u32	ezDisableButtonCallback	(u32 ButtonNumber)	// disables generation of a push button callback
{

	// use the flag service to hook the interrupt
	return((u32)adi_flag_RemoveCallback(EZ_BUTTON_TO_FLAG(ButtonNumber)));
	
}




/*********************************************************************

	Function:		ezClearButton

	Description:	Clears a push button latch.  This must be called to 
					reset the latch for the push button, if a button has
					been pressed.

*********************************************************************/

void	ezClearButton(u32 ButtonNumber)	// clears a button latch
{
	volatile int i;
	
	// delay to allow for any debounce to clear
	for (i = 0; i < 4000000; i++) ;
	
	// clear the flag corresponding to the button
	adi_flag_Clear(EZ_BUTTON_TO_FLAG(ButtonNumber));
	
}








/*********************************************************************

	Function:		ezInitPower

	Description:	Initializes and sets Power managwmentSDRAM parameters on the EZ-Kit.  
					(Replaces ezConfigurePLL & ezConfigureSDRAM )
					Processor clock set to max in each case
	
*********************************************************************/

#define DO_NOT_CHANGE_MMR_SETTINGS 0

void ezInitPower(u8 NumCores)	
{
	ADI_EBIU_RESULT EBIUResult;
	ADI_PWR_RESULT 	PWRResult;
	
#if defined (__ADSP_TETON__)
	ADI_PWR_COMMAND_PAIR ezkit_power[3];
#endif	
	
	// It is important that the EBIU module is configured before Power module
	// so that changes to the clock frequencies are correctly reflected in the
	// SDRAM settings
	
	//Initializes the EBIU module 
	ADI_EBIU_COMMAND_PAIR ezkit_sdram[] = {	
#if defined (__ADSP_EDINBURGH__)	
		{ ADI_EBIU_CMD_SET_EZKIT, (void*)ADI_EBIU_EZKIT_BF533 },
#elif defined(__ADSP_BRAEMAR__)
		{ ADI_EBIU_CMD_SET_EZKIT, (void*)ADI_EBIU_EZKIT_BF537 },
#elif defined (__ADSP_TETON__)
		{ ADI_EBIU_CMD_SET_EZKIT, (void*)ADI_EBIU_EZKIT_BF561 },
#endif
		{ ADI_EBIU_CMD_END, 0}
	};
	EBIUResult = adi_ebiu_Init( ezkit_sdram, DO_NOT_CHANGE_MMR_SETTINGS );
	if ((EBIUResult != ADI_EBIU_RESULT_SUCCESS) && (EBIUResult != ADI_EBIU_RESULT_CALL_IGNORED)) {
		ezErrorCheck(EBIUResult);
	}
	
	//Initializes the power management module 
#if defined (__ADSP_TETON__)

	u8 ic=0;
	ezkit_power[ic].kind  =  ADI_PWR_CMD_SET_EZKIT;
	ezkit_power[ic].value =  (void*)ADI_PWR_EZKIT_BF561_500MHZ;
	
	// for Teton (BF561) there is a choice between auto sync if NumCores >1 or
	// no synchronization - essential for single-core apps.
	
	if (NumCores>1) {	
		ic++;
		// value field is superfluos for this command
		ezkit_power[ic].kind  =  ADI_PWR_CMD_SET_AUTO_SYNC_ENABLED;
	}
	ic++;
	// value field is superfluos for this command
	ezkit_power[ic].kind  =  ADI_PWR_CMD_END;
	
#else	// otherwise - Edinburgh or Braemar	

	ADI_PWR_COMMAND_PAIR ezkit_power[] = {	
#if defined (__ADSP_EDINBURGH__)	
		{ ADI_PWR_CMD_SET_EZKIT, (void*)ADI_PWR_EZKIT_BF533_600MHZ },
#elif defined(__ADSP_BRAEMAR__)
		{ ADI_PWR_CMD_SET_EZKIT, (void*)ADI_PWR_EZKIT_BF537_600MHZ },
#endif
		{ ADI_PWR_CMD_END, 0}
	};	
#endif
	
	PWRResult = adi_pwr_Init( ezkit_power );
	if ((PWRResult != ADI_PWR_RESULT_SUCCESS) && (PWRResult != ADI_PWR_RESULT_CALL_IGNORED)) {
		ezErrorCheck(PWRResult);
	}
	
	// Crank up speed to max possible
#if !defined (__ADSP_TETON__) || defined(__600_MHZ_TETON__)
	ezErrorCheck( adi_pwr_SetFreq( 0, 0, ADI_PWR_DF_NONE ) );
#else
	// Again, this is safe, on boards with ADSP-BF561SKBC500X rev 0.2
	// There are issues with some BF561 EZ-kits for V >=1.2V
	ezErrorCheck( adi_pwr_SetMaxFreqForVolt( ADI_PWR_VLEV_115 ) );
#endif
}


/*********************************************************************

	Function:		ezInitAsync

	Description:	Initializes and sets the appropriate wait states for
					the async memories on the EZKit.  

*********************************************************************/

void ezInitAsync(void)						// configures async memory for use
{
	
#if defined(__ADSP_EDINBURGH__)			// ADSP-BF533 EZ-Kit specific info
	*pEBIU_AMBCTL0	= 0x7bb07bb0;	// Write access time = 7 cycles, read access time = 11 cycles, no ARDY
	*pEBIU_AMBCTL1	= 0x7bb07bb0;	// Hold time = 2 cycles, setup time = 3 cycles, transition time = 4 cycles
	*pEBIU_AMGCTL	= 0x00FF;
#endif // Edinburgh
 
}



/*********************************************************************

	Function:		ezInitFlashA

	Description:	Sets up the A flash on the board for use.  

*********************************************************************/

void ezInitFlashA(void)						// sets up the flash
{
	
#if defined(__ADSP_EDINBURGH__)			// ADSP-BF533 EZ-Kit specific info
	*pFlashA_PortA_Out = 0;			// resets port A to initial value
	*pFlashA_PortA_Dir = 0xFF;		// configure everything on port A as outputs
	*pFlashA_PortB_Out = 0;			// resets port B to initial value
	*pFlashA_PortB_Dir = 0x3f;		// configure everything on port B as outputs
#endif // Edinburgh
 	
}



/*********************************************************************

	Function:		ezInitLEDs

	Description:	Initializes and sets up the LEDs for use. 

*********************************************************************/

void ezInitLEDs(void)			// configures the flags to drive LEDs
{
	int i;

#if defined(__ADSP_EDINBURGH__)			// ADSP-BF533 EZ-Kit specific info
	// nothing to do here on the BF533 EZ-Kit
#endif // Edinburgh
	
#if defined(__ADSP_BRAEMAR__) || defined (__ADSP_TETON__) // ADSP-BF537 & BF561 EZ-Kit specific info
	// configure flags connected to LEDs as inputs
	for (i = 0; i < EZ_NUM_LEDS; i++) {
		adi_flag_Open(LED[i]);
		adi_flag_SetDirection(LED[i], ADI_FLAG_DIRECTION_OUTPUT);
	}
#endif // Braemar or Teton
 
	
}


/*********************************************************************

	Function:		ezTerminateLEDs

	Description:	Terminates the LEDs

*********************************************************************/

void ezTerminateLEDs (void)
{
	// nothing to do ?????
		
}

/*********************************************************************

	Function:		ezInitButtons

	Description:	Initializes the push button as input flags

*********************************************************************/

void ezInitButtons(void) {
	
	u32 i;

	// configure flags connected to buttons as inputs
	for (i = 0; i < EZ_NUM_BUTTONS; i++) {
		adi_flag_Open(Button[i]);
		adi_flag_SetDirection(Button[i], ADI_FLAG_DIRECTION_INPUT);
	}
		
}



/*********************************************************************

	Function:		ezTerminateButtons

	Description:	Terminates the buttons

*********************************************************************/

void ezTerminateButtons(void)
{
	// nothing to do ?????
		
}





